from typing import List, Tuple

from sqlalchemy import create_engine
from components.Model.InsumoModel import InsumoModel
from enums.DatabaseDialectEnum import DatabaseDialectEnum
from components.Databases.Enums.PostgresDataTypeEnum import PostgresDataTypeEnum
from components.Databases.Enums.PostgresDataTypeORMEnum import PostgresDataTypeORMEnum

import logging
from sqlalchemy import Table, Column, MetaData

logger = logging.getLogger()


class Postgres:

    def __init__(self, host: str, port: int, user: str, password: str, database: str, schema: str) -> None:
        self.host = host
        self.port = port
        self.user = user
        self.password = password
        self.database = database
        self.schema = schema
        self.engine = None
        self.DEFAULT_VAR_LENGHT = 100

    def create_engine(self) -> None:
        try:
            dialect = DatabaseDialectEnum.POSTGRES.value
            url = f"{dialect}://{self.user}:{self.password}@{self.host}:{str(self.port)}/{self.database}" \
                  f"?options=-csearch_path={self.schema}"
            self.engine = create_engine(url)
        except Exception as e:
            logger.error(f"Error creando engine de Postgres. {e}")

    def create_model(self, tablename: str, fields: List[Tuple[str]], modelName: str = "TableModel"):
        model = None
        try:
            model = type(modelName, (InsumoModel,), {
                '__tablename__': tablename,
                '__table_args__': {'extend_existing': True}
            })
            for field in fields:
                logger.debug(f"Attribute: {field}")
                name = field[0]
                if field[2] != -1:
                    try:
                        if not isinstance(field[3], type(None)) and field[3] > 0:
                            data_type = PostgresDataTypeORMEnum[PostgresDataTypeEnum(field[1]).name].value(precision=field[2], scale=field[3])
                        else:
                            data_type = PostgresDataTypeORMEnum[PostgresDataTypeEnum(field[1]).name].value(field[2])
                    except TypeError:
                        data_type = PostgresDataTypeORMEnum[PostgresDataTypeEnum(field[1]).name].value()
                else:
                    data_type = PostgresDataTypeORMEnum[PostgresDataTypeEnum(field[1]).name].value(self.DEFAULT_VAR_LENGHT)
                setattr(model, name, Column(data_type))
            model = model.__table__
        except Exception as e:
            logger.error(f"Error creando modelo dinámico en Postgres con nombre {tablename}. {type(e)}. {e}")
        finally:
            return model

    def generate_sql_procedure(self, command: str, reserved_word: str = "execute") -> str:
        response = ""
        try:
            command = command.replace(reserved_word, "").replace(";", "")
            response = f"call {command};"
            logger.debug("COMANDO POSTGRES:", response)
        except Exception as e:
            logger.error(f"Error generando comando sql para procedure Postgres. Comando: {command}. {e}")
        finally:
            return response
