
from airflow.providers.amazon.aws.sensors.s3 import S3KeySensor
from airflow.providers.google.cloud.sensors.gcs import GCSObjectExistenceSensor

from enums.ProviderTypeEnum import ProviderTypeEnum

import logging

logger = logging.getLogger()

POKE_INTERVAL = 5
TIMEOUT = 60*1
VERIFY_SSL = False


def create_sensor(task_id: str, connection: str, bucket: str, key: str, provider: str = "google"):
    sensor = None
    try:
        if provider == ProviderTypeEnum.GOOGLE.value:
            sensor = GCSObjectExistenceSensor(
                task_id=task_id,
                bucket=bucket,
                object=key,
                google_cloud_conn_id=connection,
                poke_interval=POKE_INTERVAL,
                timeout=TIMEOUT
            )
        else:
            sensor = S3KeySensor(
                task_id=task_id,
                bucket_key=key,
                bucket_name=bucket,
                wildcard_match=True,
                aws_conn_id=connection,
                verify=VERIFY_SSL,
                poke_interval=POKE_INTERVAL,
                timeout=TIMEOUT
            )
    except Exception as e:
        logger.error(f"Error creando Sensor S3. {e}")
    return sensor
